@@ -4,7 +4,7 @@
 use crate::fs::CloneFlags;
 use crate::{
     fs::{Access, AtFlags, Mode, OFlags, Stat},
-    path,
+    io, path,
     time::Timespec,
 };
 use io_lifetimes::{AsFd, BorrowedFd, OwnedFd};
@@ -28,40 +28,32 @@ use libc::{fstatat as libc_fstatat, openat as libc_openat};
     )
 ))]
 use libc::{fstatat64 as libc_fstatat, openat64 as libc_openat};
+use std::ffi::{CStr, OsString};
 #[cfg(unix)]
 use std::os::unix::ffi::OsStringExt;
 #[cfg(target_os = "wasi")]
 use std::os::wasi::ffi::OsStringExt;
-use std::{
-    convert::TryInto,
-    ffi::{CStr, OsString},
-    io,
-    mem::ManuallyDrop,
-};
-use unsafe_io::os::posish::{FromRawFd, RawFd};
+use unsafe_io::os::posish::RawFd;
 #[cfg(libc)]
 use {
     crate::{negone_err, zero_ok},
     std::mem::MaybeUninit,
-    unsafe_io::os::posish::AsRawFd,
+    unsafe_io::os::posish::{AsRawFd, FromRawFd},
 };
 
 /// Return a "file" which holds a handle which refers to the process current
-/// directory (`AT_FDCWD`). It is a `ManuallyDrop`, however the caller should
-/// not drop it explicitly, as it refers to an ambient authority rather than
-/// an open resource.
+/// directory (`AT_FDCWD`).
+#[allow(unsafe_code)]
 #[inline]
-pub fn cwd() -> ManuallyDrop<OwnedFd> {
+pub fn cwd() -> BorrowedFd<'static> {
     #[cfg(libc)]
     {
-        ManuallyDrop::new(unsafe { OwnedFd::from_raw_fd(libc::AT_FDCWD as RawFd) })
+        unsafe { BorrowedFd::<'static>::borrow_raw_fd(libc::AT_FDCWD as RawFd) }
     }
 
     #[cfg(linux_raw)]
     {
-        ManuallyDrop::new(unsafe {
-            OwnedFd::from_raw_fd(linux_raw_sys::general::AT_FDCWD as RawFd)
-        })
+        unsafe { BorrowedFd::<'static>::borrow_raw_fd(linux_raw_sys::general::AT_FDCWD as RawFd) }
     }
 }
 
@@ -74,7 +66,7 @@ pub fn openat<'f, P: path::Arg, Fd: AsFd<'f>>(
     mode: Mode,
 ) -> io::Result<OwnedFd> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _openat(dirfd, &path, oflags, mode)
 }
 
@@ -109,7 +101,7 @@ pub fn readlinkat<'f, P: path::Arg, Fd: AsFd<'f>>(
     reuse: OsString,
 ) -> io::Result<OsString> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _readlinkat(dirfd, &path, reuse)
 }
 
@@ -132,7 +124,7 @@ fn _readlinkat(dirfd: BorrowedFd<'_>, path: &CStr, reuse: OsString) -> io::Resul
                 buffer.capacity(),
             ))?;
 
-            let nread = nread.try_into().unwrap();
+            let nread = nread as usize;
             assert!(nread <= buffer.capacity());
             buffer.set_len(nread);
             if nread < buffer.capacity() {
@@ -160,7 +152,7 @@ fn _readlinkat(dirfd: BorrowedFd<'_>, path: &CStr, reuse: OsString) -> io::Resul
             std::slice::from_raw_parts_mut(buffer.as_mut_ptr(), buffer.capacity())
         })?;
 
-        let nread = nread.try_into().unwrap();
+        let nread = nread as usize;
         assert!(nread <= buffer.capacity());
         unsafe {
             buffer.set_len(nread);
@@ -178,7 +170,7 @@ fn _readlinkat(dirfd: BorrowedFd<'_>, path: &CStr, reuse: OsString) -> io::Resul
 #[inline]
 pub fn mkdirat<'f, P: path::Arg, Fd: AsFd<'f>>(dirfd: Fd, path: P, mode: Mode) -> io::Result<()> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _mkdirat(dirfd, &path, mode)
 }
 
@@ -210,8 +202,8 @@ pub fn linkat<'pf, 'qf, P: path::Arg, Q: path::Arg, PFd: AsFd<'pf>, QFd: AsFd<'q
 ) -> io::Result<()> {
     let old_dirfd = old_dirfd.as_fd();
     let new_dirfd = new_dirfd.as_fd();
-    let old_path = old_path.as_c_str()?;
-    let new_path = new_path.as_c_str()?;
+    let old_path = old_path.into_c_str()?;
+    let new_path = new_path.into_c_str()?;
     _linkat(old_dirfd, &old_path, new_dirfd, &new_path, flags)
 }
 
@@ -254,7 +246,7 @@ pub fn unlinkat<'f, P: path::Arg, Fd: AsFd<'f>>(
     flags: AtFlags,
 ) -> io::Result<()> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _unlinkat(dirfd, &path, flags)
 }
 
@@ -284,8 +276,8 @@ pub fn renameat<'pf, 'qf, P: path::Arg, Q: path::Arg, PFd: AsFd<'pf>, QFd: AsFd<
 ) -> io::Result<()> {
     let old_dirfd = old_dirfd.as_fd();
     let new_dirfd = new_dirfd.as_fd();
-    let old_path = old_path.as_c_str()?;
-    let new_path = new_path.as_c_str()?;
+    let old_path = old_path.into_c_str()?;
+    let new_path = new_path.into_c_str()?;
     _renameat(old_dirfd, &old_path, new_dirfd, &new_path)
 }
 
@@ -325,8 +317,8 @@ pub fn symlinkat<'f, P: path::Arg, Q: path::Arg, Fd: AsFd<'f>>(
     new_path: Q,
 ) -> io::Result<()> {
     let new_dirfd = new_dirfd.as_fd();
-    let old_path = old_path.as_c_str()?;
-    let new_path = new_path.as_c_str()?;
+    let old_path = old_path.into_c_str()?;
+    let new_path = new_path.into_c_str()?;
     _symlinkat(&old_path, new_dirfd, &new_path)
 }
 
@@ -355,7 +347,7 @@ pub fn statat<'f, P: path::Arg, Fd: AsFd<'f>>(
     flags: AtFlags,
 ) -> io::Result<Stat> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _statat(dirfd, &path, flags)
 }
 
@@ -388,7 +380,7 @@ pub fn accessat<'f, P: path::Arg, Fd: AsFd<'f>>(
     flags: AtFlags,
 ) -> io::Result<()> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _accessat(dirfd, &path, access, flags)
 }
 
@@ -428,15 +420,11 @@ fn _accessat(dirfd: BorrowedFd<'_>, path: &CStr, access: Access, flags: AtFlags)
     }
 
     if flags.bits() != linux_raw_sys::v5_11::general::AT_EACCESS {
-        return Err(io::Error::from_raw_os_error(
-            linux_raw_sys::errno::EINVAL as i32,
-        ));
+        return Err(io::Error::INVAL);
     }
 
     // TODO: Use faccessat2 in newer Linux versions.
-    Err(io::Error::from_raw_os_error(
-        linux_raw_sys::errno::ENOSYS as i32,
-    ))
+    Err(io::Error::NOSYS)
 }
 
 /// `utimensat(dirfd, path, times, flags)`
@@ -448,7 +436,7 @@ pub fn utimensat<'f, P: path::Arg, Fd: AsFd<'f>>(
     flags: AtFlags,
 ) -> io::Result<()> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _utimensat(dirfd, &path, times, flags)
 }
 
@@ -493,7 +481,7 @@ fn _utimensat(
 #[inline]
 pub fn chmodat<'f, P: path::Arg, Fd: AsFd<'f>>(dirfd: Fd, path: P, mode: Mode) -> io::Result<()> {
     let dirfd = dirfd.as_fd();
-    let path = path.as_c_str()?;
+    let path = path.into_c_str()?;
     _chmodat(dirfd, &path, mode)
 }
 
@@ -543,7 +531,7 @@ pub fn fclonefileat<'f, 'dst_f, Fd: AsFd<'f>, DstFd: AsFd<'dst_f>, P: path::Arg>
 ) -> io::Result<()> {
     let srcfd = src.as_fd();
     let dst_dirfd = dst_dir.as_fd();
-    let dst = dst.as_c_str()?;
+    let dst = dst.into_c_str()?;
     _fclonefileat(srcfd.as_fd(), dst_dirfd.as_fd(), &dst, flags)
 }
 
