@@ -1,16 +1,19 @@
 //! I/O operations.
 
-mod errno;
+mod error;
 mod fd;
 mod ioctl;
+mod pipe;
 mod poll;
 #[cfg(any(target_os = "macos", target_os = "ios"))]
 mod rdadvise;
 mod read_write;
 #[cfg(not(target_os = "wasi"))]
 mod socketpair;
+#[cfg(any(target_os = "android", target_os = "linux"))]
+mod userfaultfd;
 
-pub use crate::io::errno::Errno;
+pub use error::{Error, Result};
 #[cfg(not(target_os = "wasi"))]
 pub use fd::dup;
 #[cfg(not(target_os = "redox"))]
@@ -23,7 +26,11 @@ pub use fd::ttyname;
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 pub use ioctl::ioctl_fioclex;
 #[cfg(not(target_os = "wasi"))]
-pub use ioctl::ioctl_tcgets;
+pub use ioctl::{ioctl_tcgets, ioctl_tiocgwinsz};
+#[cfg(any(target_os = "ios", target_os = "macos"))]
+pub use pipe::pipe;
+#[cfg(not(any(target_os = "ios", target_os = "macos", target_os = "wasi")))]
+pub use pipe::{pipe2, PipeFlags};
 pub use poll::{PollFd, PollFdVec, PollFlags};
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 pub use rdadvise::rdadvise;
@@ -34,6 +41,8 @@ pub use read_write::{preadv, pwritev};
 pub use read_write::{preadv2, pwritev2};
 #[cfg(not(target_os = "wasi"))]
 pub use socketpair::socketpair_stream;
+#[cfg(any(target_os = "android", target_os = "linux"))]
+pub use userfaultfd::{userfaultfd, UserFaultFdFlags};
 
 /// Re-export `termios`.
 #[cfg(all(libc, not(target_os = "wasi")))]
@@ -43,6 +52,14 @@ pub type Termios = libc::termios;
 #[cfg(linux_raw)]
 pub type Termios = linux_raw_sys::general::termios;
 
+/// Re-export `winsize`.
+#[cfg(all(libc, not(target_os = "wasi")))]
+pub type Winsize = libc::winsize;
+
+/// Re-export `winsize`.
+#[cfg(linux_raw)]
+pub type Winsize = linux_raw_sys::general::winsize;
+
 /// Re-export `ICANON`.
 #[cfg(all(libc, not(target_os = "wasi")))]
 pub const ICANON: libc::tcflag_t = libc::ICANON;
