@@ -1,6 +1,7 @@
+use crate::io;
 use bitflags::bitflags;
 use io_lifetimes::{AsFd, BorrowedFd};
-use std::{io, marker::PhantomData, vec::IntoIter};
+use std::{marker::PhantomData, vec::IntoIter};
 #[cfg(libc)]
 use {crate::negone_err, std::convert::TryInto, unsafe_io::os::posish::AsRawFd};
 
@@ -33,6 +34,11 @@ bitflags! {
         const HUP = libc::POLLHUP;
         /// `POLLNVAL`
         const NVAL = libc::POLLNVAL;
+        /// `POLLRDHUP`
+        // TODO: Submitted to upstream libc:
+        // <https://github.com/rust-lang/libc/pull/2247>
+        #[cfg(not(any(target_arch = "sparc", target_arch = "sparc64")))]
+        const RDHUP = 0x2000;
     }
 }
 
@@ -60,6 +66,8 @@ bitflags! {
         const HUP = linux_raw_sys::general::POLLHUP as u16;
         /// `POLLNVAL`
         const NVAL = linux_raw_sys::general::POLLNVAL as u16;
+        /// `POLLRDHUP`
+        const RDHUP = linux_raw_sys::general::POLLRDHUP as u16;
     }
 }
 
@@ -95,7 +103,7 @@ impl<'fd> PollFd<'fd> {
             pollfd: libc::pollfd {
                 fd: fd.as_raw_fd() as libc::c_int,
                 events: events.bits(),
-                revents: PollFlags::empty().bits(),
+                revents: 0,
             },
             _phantom: PhantomData,
         }
@@ -117,6 +125,8 @@ impl<'fd> PollFd<'fd> {
     /// Return the ready events.
     #[inline]
     pub fn revents(self) -> PollFlags {
+        // Use `unwrap()` here because in theory we know we know all the bits
+        // the OS might set here, but OS's have added extensions in the past.
         PollFlags::from_bits(self.pollfd.revents).unwrap()
     }
 }
@@ -157,12 +167,12 @@ impl<'fd> PollFdVec<'fd> {
             .fds
             .len()
             .try_into()
-            .map_err(|_convert_err| io::Error::from_raw_os_error(libc::EINVAL))?;
+            .map_err(|_convert_err| io::Error::INVAL)?;
 
         let nready =
             negone_err(unsafe { libc::poll(self.fds.as_mut_ptr() as *mut _, nfds, timeout) })?;
 
-        Ok(nready.try_into().unwrap())
+        Ok(nready as usize)
     }
 }
 
@@ -170,7 +180,7 @@ impl<'fd> PollFdVec<'fd> {
 impl<'fd> PollFdVec<'fd> {
     /// `poll(self.fds.as_mut_ptr(), self.fds.len(), timeout)`
     pub fn poll(&mut self, timeout: std::os::raw::c_int) -> io::Result<usize> {
-        // `Pollfd` is `repr(transparent)` so we can transmute slices of it.
+        // `PollFd` is `repr(transparent)` so we can transmute slices of it.
         crate::linux_raw::poll(
             unsafe { std::mem::transmute(self.fds.as_mut_slice()) },
             timeout,
