@@ -1,5 +1,6 @@
 //! `read` and `write`, optionally positioned, optionally vectored
 
+use crate::io;
 #[cfg(any(linux_raw, all(libc, target_os = "linux", target_env = "gnu")))]
 use bitflags::bitflags;
 use io_lifetimes::{AsFd, BorrowedFd};
@@ -35,10 +36,10 @@ use std::sync::atomic::{AtomicUsize, Ordering};
 use std::{
     cmp::min,
     convert::TryInto,
-    io::{self, IoSlice, IoSliceMut},
+    io::{IoSlice, IoSliceMut},
 };
 #[cfg(libc)]
-use {crate::negone_err, unsafe_io::os::posish::AsRawFd};
+use {crate::negone_err, std::os::raw::c_int, unsafe_io::os::posish::AsRawFd};
 
 #[cfg(all(libc, target_os = "linux", target_env = "gnu"))]
 bitflags! {
@@ -90,7 +91,7 @@ fn _read(fd: BorrowedFd<'_>, buf: &mut [u8]) -> io::Result<usize> {
             buf.len(),
         ))?
     };
-    Ok(nread.try_into().unwrap())
+    Ok(nread as usize)
 }
 
 #[cfg(linux_raw)]
@@ -115,7 +116,7 @@ fn _write(fd: BorrowedFd<'_>, buf: &[u8]) -> io::Result<usize> {
             buf.len(),
         ))?
     };
-    Ok(nwritten.try_into().unwrap())
+    Ok(nwritten as usize)
 }
 
 #[cfg(linux_raw)]
@@ -135,7 +136,7 @@ pub fn pread<'f, Fd: AsFd<'f>>(fd: Fd, buf: &mut [u8], offset: u64) -> io::Resul
 fn _pread(fd: BorrowedFd<'_>, buf: &mut [u8], offset: u64) -> io::Result<usize> {
     let offset = offset
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let nread = unsafe {
         negone_err(libc_pread(
             fd.as_raw_fd() as libc::c_int,
@@ -144,7 +145,7 @@ fn _pread(fd: BorrowedFd<'_>, buf: &mut [u8], offset: u64) -> io::Result<usize>
             offset,
         ))?
     };
-    Ok(nread.try_into().unwrap())
+    Ok(nread as usize)
 }
 
 #[cfg(linux_raw)]
@@ -164,7 +165,7 @@ pub fn pwrite<'f, Fd: AsFd<'f>>(fd: Fd, buf: &[u8], offset: u64) -> io::Result<u
 fn _pwrite(fd: BorrowedFd<'_>, buf: &[u8], offset: u64) -> io::Result<usize> {
     let offset = offset
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let nwritten = unsafe {
         negone_err(libc_pwrite(
             fd.as_raw_fd() as libc::c_int,
@@ -173,7 +174,7 @@ fn _pwrite(fd: BorrowedFd<'_>, buf: &[u8], offset: u64) -> io::Result<usize> {
             offset,
         ))?
     };
-    Ok(nwritten.try_into().unwrap())
+    Ok(nwritten as usize)
 }
 
 #[cfg(linux_raw)]
@@ -195,10 +196,10 @@ fn _readv(fd: BorrowedFd<'_>, bufs: &[IoSliceMut]) -> io::Result<usize> {
         negone_err(libc_readv(
             fd.as_raw_fd() as libc::c_int,
             bufs.as_ptr().cast::<libc::iovec>(),
-            min(bufs.len(), max_iov()).try_into().unwrap(),
+            min(bufs.len(), max_iov()) as c_int,
         ))?
     };
-    Ok(nread.try_into().unwrap())
+    Ok(nread as usize)
 }
 
 #[cfg(linux_raw)]
@@ -220,10 +221,10 @@ fn _writev(fd: BorrowedFd<'_>, bufs: &[IoSlice]) -> io::Result<usize> {
         negone_err(libc_writev(
             fd.as_raw_fd() as libc::c_int,
             bufs.as_ptr().cast::<libc::iovec>(),
-            min(bufs.len(), max_iov()).try_into().unwrap(),
+            min(bufs.len(), max_iov()) as c_int,
         ))?
     };
-    Ok(nwritten.try_into().unwrap())
+    Ok(nwritten as usize)
 }
 
 #[cfg(linux_raw)]
@@ -244,24 +245,24 @@ pub fn preadv<'f, Fd: AsFd<'f>>(fd: Fd, bufs: &[IoSliceMut], offset: u64) -> io:
 fn _preadv(fd: BorrowedFd<'_>, bufs: &[IoSliceMut], offset: u64) -> io::Result<usize> {
     let offset = offset
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let nread = unsafe {
         negone_err(libc_preadv(
             fd.as_raw_fd() as libc::c_int,
             bufs.as_ptr().cast::<libc::iovec>(),
-            min(bufs.len(), max_iov()).try_into().unwrap(),
+            min(bufs.len(), max_iov()) as c_int,
             offset,
         ))?
     };
-    Ok(nread.try_into().unwrap())
+    Ok(nread as usize)
 }
 
 #[cfg(linux_raw)]
 #[inline]
 fn _preadv(fd: BorrowedFd<'_>, bufs: &[IoSliceMut], offset: u64) -> io::Result<usize> {
-    let offset = offset.try_into().map_err(|_overflow_err| {
-        io::Error::from_raw_os_error(linux_raw_sys::errno::EOVERFLOW as i32)
-    })?;
+    let offset = offset
+        .try_into()
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     crate::linux_raw::preadv(fd, &bufs[..min(bufs.len(), max_iov())], offset)
 }
 
@@ -277,24 +278,24 @@ pub fn pwritev<'f, Fd: AsFd<'f>>(fd: Fd, bufs: &[IoSlice], offset: u64) -> io::R
 fn _pwritev(fd: BorrowedFd<'_>, bufs: &[IoSlice], offset: u64) -> io::Result<usize> {
     let offset = offset
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let nwritten = unsafe {
         negone_err(libc_pwritev(
             fd.as_raw_fd() as libc::c_int,
             bufs.as_ptr().cast::<libc::iovec>(),
-            min(bufs.len(), max_iov()).try_into().unwrap(),
+            min(bufs.len(), max_iov()) as c_int,
             offset,
         ))?
     };
-    Ok(nwritten.try_into().unwrap())
+    Ok(nwritten as usize)
 }
 
 #[cfg(linux_raw)]
 #[inline]
 fn _pwritev(fd: BorrowedFd<'_>, bufs: &[IoSlice], offset: u64) -> io::Result<usize> {
-    let offset = offset.try_into().map_err(|_overflow_err| {
-        io::Error::from_raw_os_error(linux_raw_sys::errno::EOVERFLOW as i32)
-    })?;
+    let offset = offset
+        .try_into()
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     crate::linux_raw::pwritev(fd, &bufs[..min(bufs.len(), max_iov())], offset)
 }
 
@@ -320,17 +321,17 @@ fn _preadv2(
 ) -> io::Result<usize> {
     let offset = offset
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let nread = unsafe {
         negone_err(libc_preadv2(
             fd.as_raw_fd() as libc::c_int,
             bufs.as_ptr().cast::<libc::iovec>(),
-            min(bufs.len(), max_iov()).try_into().unwrap(),
+            min(bufs.len(), max_iov()) as c_int,
             offset,
             flags.bits(),
         ))?
     };
-    Ok(nread.try_into().unwrap())
+    Ok(nread as usize)
 }
 
 #[cfg(linux_raw)]
@@ -341,9 +342,9 @@ fn _preadv2(
     offset: u64,
     flags: ReadWriteFlags,
 ) -> io::Result<usize> {
-    let offset = offset.try_into().map_err(|_overflow_err| {
-        io::Error::from_raw_os_error(linux_raw_sys::errno::EOVERFLOW as i32)
-    })?;
+    let offset = offset
+        .try_into()
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     crate::linux_raw::preadv2(
         fd,
         &bufs[..min(bufs.len(), max_iov())],
@@ -374,17 +375,17 @@ fn _pwritev2(
 ) -> io::Result<usize> {
     let offset = offset
         .try_into()
-        .map_err(|_overflow_err| io::Error::from_raw_os_error(libc::EOVERFLOW))?;
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     let nwritten = unsafe {
         negone_err(libc_pwritev2(
             fd.as_raw_fd() as libc::c_int,
             bufs.as_ptr().cast::<libc::iovec>(),
-            min(bufs.len(), max_iov()).try_into().unwrap(),
+            min(bufs.len(), max_iov()) as c_int,
             offset,
             flags.bits(),
         ))?
     };
-    Ok(nwritten.try_into().unwrap())
+    Ok(nwritten as usize)
 }
 
 #[cfg(linux_raw)]
@@ -395,9 +396,9 @@ fn _pwritev2(
     offset: u64,
     flags: ReadWriteFlags,
 ) -> io::Result<usize> {
-    let offset = offset.try_into().map_err(|_overflow_err| {
-        io::Error::from_raw_os_error(linux_raw_sys::errno::EOVERFLOW as i32)
-    })?;
+    let offset = offset
+        .try_into()
+        .map_err(|_overflow_err| io::Error::OVERFLOW)?;
     crate::linux_raw::pwritev2(
         fd,
         &bufs[..min(bufs.len(), max_iov())],
@@ -410,22 +411,31 @@ fn _pwritev2(
 // revision 108e90ca78f052c0c1c49c42a22c85620be19712.
 
 #[cfg(all(libc, not(any(target_os = "redox", target_env = "newlib"))))]
-fn max_iov() -> usize {
-    static LIM: AtomicUsize = AtomicUsize::new(0);
+static LIM: AtomicUsize = AtomicUsize::new(0);
 
+#[cfg(all(libc, not(any(target_os = "redox", target_env = "newlib"))))]
+#[inline]
+fn max_iov() -> usize {
     let mut lim = LIM.load(Ordering::Relaxed);
     if lim == 0 {
-        let ret = unsafe { libc::sysconf(libc::_SC_IOV_MAX) };
-
-        // 16 is the minimum value required by POSIX.
-        lim = if ret > 0 { ret as usize } else { 16 };
-        LIM.store(lim, Ordering::Relaxed);
+        lim = query_max_iov()
     }
 
     lim
 }
 
+#[cfg(all(libc, not(any(target_os = "redox", target_env = "newlib"))))]
+fn query_max_iov() -> usize {
+    let ret = unsafe { libc::sysconf(libc::_SC_IOV_MAX) };
+
+    // 16 is the minimum value required by POSIX.
+    let lim = if ret > 0 { ret as usize } else { 16 };
+    LIM.store(lim, Ordering::Relaxed);
+    lim
+}
+
 #[cfg(all(libc, any(target_os = "redox", target_env = "newlib")))]
+#[inline]
 fn max_iov() -> usize {
     16 // The minimum value required by POSIX.
 }
