@@ -1,6 +1,5 @@
-#[cfg(linux_raw)]
-use io_lifetimes::FromFd;
-use std::{convert::TryInto, io, net::TcpStream};
+use crate::io;
+use io_lifetimes::OwnedFd;
 #[cfg(libc)]
 use {
     crate::zero_ok,
@@ -10,7 +9,7 @@ use {
 
 /// `socketpair(domain, SOCK_STREAM | SOCK_CLOEXEC, protocol)`
 #[cfg(libc)]
-pub fn socketpair_stream(domain: i32, protocol: i32) -> io::Result<(TcpStream, TcpStream)> {
+pub fn socketpair_stream(domain: i32, protocol: i32) -> io::Result<(OwnedFd, OwnedFd)> {
     let mut fds = MaybeUninit::<[RawFd; 2]>::uninit();
     unsafe {
         #[cfg(not(any(target_os = "ios", target_os = "macos")))]
@@ -21,9 +20,9 @@ pub fn socketpair_stream(domain: i32, protocol: i32) -> io::Result<(TcpStream, T
         let flags = 0;
 
         zero_ok(libc::socketpair(
-            domain.try_into().unwrap(),
+            domain,
             libc::SOCK_STREAM | flags,
-            protocol.try_into().unwrap(),
+            protocol,
             fds.as_mut_ptr().cast::<RawFd>(),
         ))?;
 
@@ -35,20 +34,17 @@ pub fn socketpair_stream(domain: i32, protocol: i32) -> io::Result<(TcpStream, T
             zero_ok(libc::ioctl(*fd, libc::FIOCLEX))?;
         }
 
-        Ok((
-            TcpStream::from_raw_fd(fds[0]),
-            TcpStream::from_raw_fd(fds[1]),
-        ))
+        Ok((OwnedFd::from_raw_fd(fds[0]), OwnedFd::from_raw_fd(fds[1])))
     }
 }
 
 /// `socketpair(domain, SOCK_STREAM | SOCK_CLOEXEC, protocol)`
 #[cfg(linux_raw)]
-pub fn socketpair_stream(domain: i32, protocol: i32) -> io::Result<(TcpStream, TcpStream)> {
+pub fn socketpair_stream(domain: i32, protocol: i32) -> io::Result<(OwnedFd, OwnedFd)> {
     crate::linux_raw::socketpair(
-        domain.try_into().unwrap(),
+        domain,
         linux_raw_sys::general::SOCK_STREAM | linux_raw_sys::general::SOCK_CLOEXEC,
         protocol,
     )
-    .map(|fds| (TcpStream::from_fd(fds.0), TcpStream::from_fd(fds.1)))
+    .map(|fds| (fds.0, fds.1))
 }
