@@ -20,16 +20,15 @@
 //!    are used.
 //!
 //! Things they don't do include:
-//!  - Emulating functions that aren't natively supported on a platform.
 //!  - Detecting whether functions are supported at runtime.
 //!  - Hiding significant differences between platforms.
-//!  - Hiding ambient authorities.
+//!  - Restricting ambient authorities.
 //!  - Imposing sandboxing features such as filesystem path or network address
 //!    sandboxing.
 //!
-//! See [`cap-std`] and [`system-interface`] for libraries which do hide
-//! ambient authorities, perform sandboxing, and hide significant differences
-//! between platforms.
+//! See [`cap-std`], [`system-interface`], and [`io-streams`] for libraries
+//! which do hide significant differences between platforms, and [`cap-std`]
+//! which does perform sandboxing and restricts ambient authorities.
 //!
 //! # Safety
 //!
@@ -38,6 +37,7 @@
 //!
 //! [`cap-std`]: https://crates.io/crates/cap-std
 //! [`system-interface`]: https://crates.io/crates/system-interface
+//! [`io-streams`]: https://crates.io/crates/io-streams
 //! [`std`]: https://doc.rust-lang.org/std/
 //! [`getrandom`]: https://crates.io/crates/getrandom
 //! [`std::fs::File`]: https://doc.rust-lang.org/std/fs/struct.File.html
@@ -60,6 +60,7 @@ pub mod io;
 pub mod net;
 pub mod path;
 pub mod process;
+pub mod rand;
 pub mod time;
 
 #[cfg(linux_raw)]
@@ -68,20 +69,20 @@ mod linux_raw;
 /// Given a `libc` return value, translate `0` into `Ok(())` and any other
 /// value to an `Err` with the error from `errno`.
 #[cfg(libc)]
-fn zero_ok<T: LibcResult>(t: T) -> std::io::Result<()> {
+fn zero_ok<T: LibcResult>(t: T) -> crate::io::Result<()> {
     if t.is_zero() {
         Ok(())
     } else {
-        Err(std::io::Error::last_os_error())
+        Err(crate::io::Error::last_os_error())
     }
 }
 
 /// Given a `libc` return value, translate `-1` into an `Err` with the error
 /// from `errno`, and any other value to an `Ok` containing the value.
 #[cfg(libc)]
-fn negone_err<T: LibcResult>(t: T) -> std::io::Result<T> {
+fn negone_err<T: LibcResult>(t: T) -> crate::io::Result<T> {
     if t.is_negone() {
-        Err(std::io::Error::last_os_error())
+        Err(crate::io::Error::last_os_error())
     } else {
         Ok(t)
     }
@@ -92,9 +93,9 @@ fn negone_err<T: LibcResult>(t: T) -> std::io::Result<T> {
 /// value.
 #[cfg(libc)]
 #[allow(dead_code)]
-fn negative_err<T: LibcResult>(t: T) -> std::io::Result<()> {
+fn negative_err<T: LibcResult>(t: T) -> crate::io::Result<()> {
     if t.is_negative() {
-        Err(std::io::Error::last_os_error())
+        Err(crate::io::Error::last_os_error())
     } else {
         Ok(())
     }
