@@ -3,6 +3,7 @@
 mod arch;
 mod conv;
 
+use crate::io;
 #[cfg(target_pointer_width = "64")]
 use crate::linux_raw::conv::ret_u64;
 use arch::{
@@ -32,8 +33,8 @@ use linux_raw_sys::{
     },
     general::{
         __kernel_clockid_t, __kernel_gid_t, __kernel_loff_t, __kernel_pid_t,
-        __kernel_sockaddr_storage, __kernel_uid_t, socklen_t, stat, statfs64, termios, timespec,
-        umode_t, winsize,
+        __kernel_sockaddr_storage, __kernel_uid_t, sockaddr_in, sockaddr_in6, sockaddr_un,
+        socklen_t, stat, statfs64, termios, timespec, umode_t, winsize,
     },
     general::{
         AT_FDCWD, AT_REMOVEDIR, AT_SYMLINK_NOFOLLOW, FIONREAD, F_DUPFD, F_DUPFD_CLOEXEC, F_GETFD,
@@ -45,14 +46,14 @@ use linux_raw_sys::{
         general::statx,
         general::{
             __NR_copy_file_range, __NR_getrandom, __NR_memfd_create, __NR_preadv2, __NR_pwritev2,
-            __NR_statx,
+            __NR_statx, __NR_userfaultfd,
         },
         general::{F_GETPIPE_SZ, F_GET_SEALS, F_SETPIPE_SZ},
     },
 };
 use std::{
     ffi::CStr,
-    io::{self, IoSlice, IoSliceMut},
+    io::{IoSlice, IoSliceMut},
     mem::{size_of, MaybeUninit},
     os::raw::{c_char, c_int, c_uint, c_void},
 };
@@ -69,7 +70,9 @@ pub(crate) fn exit_group(code: c_int) -> ! {
 
 #[inline]
 pub(crate) fn close(fd: OwnedFd) {
-    let _ = unsafe { syscall1_readonly(__NR_close, owned_fd(fd)) };
+    unsafe {
+        let _ = syscall1_readonly(__NR_close, owned_fd(fd));
+    }
 }
 
 #[inline]
@@ -411,7 +414,7 @@ pub(crate) fn fchmod(fd: BorrowedFd<'_>, mode: umode_t) -> io::Result<()> {
 }
 
 #[inline]
-pub(crate) fn lseek(fd: BorrowedFd<'_>, offset: i64, whence: c_uint) -> io::Result<u64> {
+pub(crate) fn seek(fd: BorrowedFd<'_>, offset: i64, whence: c_uint) -> io::Result<u64> {
     #[cfg(target_pointer_width = "32")]
     {
         let mut result = MaybeUninit::uninit();
@@ -682,6 +685,15 @@ pub(crate) fn fcntl_dupfd(fd: BorrowedFd<'_>) -> io::Result<OwnedFd> {
 
 #[inline]
 pub(crate) fn fcntl_dupfd_cloexec(fd: BorrowedFd<'_>) -> io::Result<OwnedFd> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_owned_fd(syscall2_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_DUPFD_CLOEXEC),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_owned_fd(syscall2_readonly(
             __NR_fcntl,
@@ -693,6 +705,15 @@ pub(crate) fn fcntl_dupfd_cloexec(fd: BorrowedFd<'_>) -> io::Result<OwnedFd> {
 
 #[inline]
 pub(crate) fn fcntl_getfd(fd: BorrowedFd<'_>) -> io::Result<c_uint> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_c_uint(syscall2_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_GETFD),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_c_uint(syscall2_readonly(
             __NR_fcntl,
@@ -704,6 +725,16 @@ pub(crate) fn fcntl_getfd(fd: BorrowedFd<'_>) -> io::Result<c_uint> {
 
 #[inline]
 pub(crate) fn fcntl_setfd(fd: BorrowedFd<'_>, flags: c_uint) -> io::Result<()> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret(syscall3_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_SETFD),
+            c_uint(flags),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret(syscall3_readonly(
             __NR_fcntl,
@@ -716,6 +747,15 @@ pub(crate) fn fcntl_setfd(fd: BorrowedFd<'_>, flags: c_uint) -> io::Result<()> {
 
 #[inline]
 pub(crate) fn fcntl_getfl(fd: BorrowedFd<'_>) -> io::Result<c_uint> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_c_uint(syscall2_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_GETFL),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_c_uint(syscall2_readonly(
             __NR_fcntl,
@@ -727,6 +767,16 @@ pub(crate) fn fcntl_getfl(fd: BorrowedFd<'_>) -> io::Result<c_uint> {
 
 #[inline]
 pub(crate) fn fcntl_setfl(fd: BorrowedFd<'_>, flags: c_uint) -> io::Result<()> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret(syscall3_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_SETFL),
+            c_uint(flags),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret(syscall3_readonly(
             __NR_fcntl,
@@ -739,6 +789,15 @@ pub(crate) fn fcntl_setfl(fd: BorrowedFd<'_>, flags: c_uint) -> io::Result<()> {
 
 #[inline]
 pub(crate) fn fcntl_getlease(fd: BorrowedFd<'_>) -> io::Result<c_int> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_c_int(syscall2_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_GETLEASE),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_c_int(syscall2_readonly(
             __NR_fcntl,
@@ -750,6 +809,15 @@ pub(crate) fn fcntl_getlease(fd: BorrowedFd<'_>) -> io::Result<c_int> {
 
 #[inline]
 pub(crate) fn fcntl_getown(fd: BorrowedFd<'_>) -> io::Result<c_int> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_c_int(syscall2_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_GETOWN),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_c_int(syscall2_readonly(
             __NR_fcntl,
@@ -761,6 +829,15 @@ pub(crate) fn fcntl_getown(fd: BorrowedFd<'_>) -> io::Result<c_int> {
 
 #[inline]
 pub(crate) fn fcntl_getsig(fd: BorrowedFd<'_>) -> io::Result<c_int> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_c_int(syscall2_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_GETSIG),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_c_int(syscall2_readonly(
             __NR_fcntl,
@@ -772,6 +849,15 @@ pub(crate) fn fcntl_getsig(fd: BorrowedFd<'_>) -> io::Result<c_int> {
 
 #[inline]
 pub(crate) fn fcntl_getpipe_sz(fd: BorrowedFd<'_>) -> io::Result<usize> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_usize(syscall2_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_GETPIPE_SZ),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_usize(syscall2_readonly(
             __NR_fcntl,
@@ -783,6 +869,16 @@ pub(crate) fn fcntl_getpipe_sz(fd: BorrowedFd<'_>) -> io::Result<usize> {
 
 #[inline]
 pub(crate) fn fcntl_setpipe_sz(fd: BorrowedFd<'_>, size: c_int) -> io::Result<usize> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_usize(syscall3_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_SETPIPE_SZ),
+            c_int(size),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_usize(syscall3_readonly(
             __NR_fcntl,
@@ -795,6 +891,15 @@ pub(crate) fn fcntl_setpipe_sz(fd: BorrowedFd<'_>, size: c_int) -> io::Result<us
 
 #[inline]
 pub(crate) fn fcntl_get_seals(fd: BorrowedFd<'_>) -> io::Result<c_int> {
+    #[cfg(target_pointer_width = "32")]
+    unsafe {
+        ret_c_int(syscall2_readonly(
+            __NR_fcntl64,
+            borrowed_fd(fd),
+            c_uint(F_GET_SEALS),
+        ))
+    }
+    #[cfg(target_pointer_width = "64")]
     unsafe {
         ret_c_int(syscall2_readonly(
             __NR_fcntl,
@@ -966,15 +1071,25 @@ pub(crate) fn getdents(fd: BorrowedFd<'_>, dirent: &mut [u8]) -> io::Result<usiz
 }
 
 #[inline]
-pub(crate) fn pipe2(filedes: &mut [OwnedFd; 2], flags: c_int) -> io::Result<()> {
-    unsafe { ret(syscall2(__NR_pipe2, slice_addr(filedes), c_int(flags))) }
+pub(crate) fn pipe2(flags: c_uint) -> io::Result<(OwnedFd, OwnedFd)> {
+    #[cfg(any(target_arch = "mips", target_arch = "mips64"))]
+    {
+        todo!("On MIPS pipe returns multiple values")
+    }
+    #[cfg(not(any(target_arch = "mips", target_arch = "mips64")))]
+    unsafe {
+        let mut result = MaybeUninit::<[OwnedFd; 2]>::uninit();
+        ret(syscall2(__NR_pipe2, out(&mut result), c_uint(flags)))?;
+        let [p0, p1] = result.assume_init();
+        Ok((p0, p1))
+    }
 }
 
 #[inline]
-pub(crate) fn pipe(filedes: &mut [OwnedFd; 2]) -> io::Result<()> {
+pub(crate) fn pipe() -> io::Result<(OwnedFd, OwnedFd)> {
     #[cfg(target_arch = "aarch64")]
     {
-        pipe2(filedes, 0)
+        pipe2(fd, 0)
     }
     #[cfg(any(target_arch = "mips", target_arch = "mips64"))]
     {
@@ -982,7 +1097,10 @@ pub(crate) fn pipe(filedes: &mut [OwnedFd; 2]) -> io::Result<()> {
     }
     #[cfg(not(any(target_arch = "aarch64", target_arch = "mips", target_arch = "mips64")))]
     unsafe {
-        ret(syscall1(__NR_pipe, slice_addr(filedes)))
+        let mut result = MaybeUninit::<[OwnedFd; 2]>::uninit();
+        ret(syscall1(__NR_pipe, out(&mut result)))?;
+        let [p0, p1] = result.assume_init();
+        Ok((p0, p1))
     }
 }
 
@@ -999,14 +1117,14 @@ pub(crate) fn getrandom(buf: &mut [u8], flags: c_uint) -> io::Result<usize> {
 }
 
 #[inline]
-pub(crate) fn socket(family: c_int, typ: c_int, protocol: c_int) -> io::Result<OwnedFd> {
+pub(crate) fn socket(family: c_uint, typ: c_uint, protocol: c_uint) -> io::Result<OwnedFd> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
         ret_owned_fd(syscall3_readonly(
             __NR_socket,
-            c_int(family),
-            c_int(typ),
-            c_int(protocol),
+            c_uint(family),
+            c_uint(typ),
+            c_uint(protocol),
         ))
     }
     #[cfg(target_arch = "x86")]
@@ -1014,7 +1132,7 @@ pub(crate) fn socket(family: c_int, typ: c_int, protocol: c_int) -> io::Result<O
         ret_owned_fd(syscall2_readonly(
             __NR_socketcall,
             SYS_SOCKET,
-            slice_addr(&[c_int(family), c_int(typ), c_int(protocol), 0, 0, 0]),
+            slice_addr(&[c_uint(family), c_uint(typ), c_uint(protocol), 0, 0, 0]),
         ))
     }
 }
@@ -1065,92 +1183,89 @@ pub(crate) fn socketpair(
 #[inline]
 pub(crate) fn accept(
     fd: BorrowedFd<'_>,
-    addr: &mut __kernel_sockaddr_storage,
-    addrlen: &mut socklen_t,
-) -> io::Result<OwnedFd> {
+) -> io::Result<(OwnedFd, __kernel_sockaddr_storage, socklen_t)> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
-        ret_owned_fd(syscall3(
+        let mut addrlen = 0;
+        let mut storage = MaybeUninit::uninit();
+        let fd = ret_owned_fd(syscall3(
             __NR_accept,
             borrowed_fd(fd),
-            by_mut(addr),
-            by_mut(addrlen),
-        ))
+            out(&mut storage),
+            by_mut(&mut addrlen),
+        ))?;
+        Ok((fd, storage.assume_init(), addrlen))
     }
     #[cfg(target_arch = "x86")]
     unsafe {
-        ret_owned_fd(syscall2(
+        let mut addrlen = 0;
+        let mut storage = MaybeUninit::uninit();
+        let fd = ret_owned_fd(syscall2(
             __NR_socketcall,
             SYS_ACCEPT,
-            slice_addr(&[borrwed_fd(fd), by_mut(addr), by_mut(addrlen), 0, 0, 0]),
-        ))
+            slice_addr(&[
+                borrwed_fd(fd),
+                out(&mut storage),
+                by_mut(&mut addrlen),
+                0,
+                0,
+                0,
+            ]),
+        ))?;
+        Ok((fd, storage.assume_init(), addrlen))
     }
 }
 
 #[inline]
 pub(crate) fn accept4(
     fd: BorrowedFd<'_>,
-    addr: &mut __kernel_sockaddr_storage,
-    addrlen: &mut socklen_t,
-    flags: c_int,
-) -> io::Result<OwnedFd> {
+    flags: c_uint,
+) -> io::Result<(OwnedFd, __kernel_sockaddr_storage, socklen_t)> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
-        ret_owned_fd(syscall4(
+        let mut addrlen = 0;
+        let mut storage = MaybeUninit::uninit();
+        let fd = ret_owned_fd(syscall4(
             __NR_accept4,
             borrowed_fd(fd),
-            by_ref(addr),
-            by_mut(addrlen),
-            c_int(flags),
-        ))
+            out(&mut storage),
+            by_mut(&mut addrlen),
+            c_uint(flags),
+        ))?;
+        Ok((fd, storage.assume_init(), addrlen))
     }
     #[cfg(target_arch = "x86")]
     unsafe {
-        ret_owned_fd(syscall2(
+        let mut addrlen = 0;
+        let mut storage = MaybeUninit::uninit();
+        let fd = ret_owned_fd(syscall2(
             __NR_socketcall,
             SYS_ACCEPT4,
-            slice_addr(&[borrwed_fd(fd), by_mut(addr), by_mut(addrlen), flags, 0, 0]),
-        ))
+            slice_addr(&[
+                borrwed_fd(fd),
+                out(&mut storage),
+                by_mut(&mut addrlen),
+                c_uint(flags),
+                0,
+                0,
+            ]),
+        ));
+        Ok((fd, storage.assume_init(), addrlen))
     }
 }
 
 #[inline]
-pub(crate) fn connect(
-    fd: BorrowedFd<'_>,
-    addr: &__kernel_sockaddr_storage,
-    addrlen: socklen_t,
-) -> io::Result<()> {
+pub(crate) fn shutdown(fd: BorrowedFd<'_>, how: c_uint) -> io::Result<()> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
-        ret(syscall3_readonly(
-            __NR_connect,
-            borrowed_fd(fd),
-            by_ref(addr),
-            socklen_t(addrlen),
-        ))
-    }
-    #[cfg(target_arch = "x86")]
-    unsafe {
-        ret(syscall2_readonly(
-            __NR_socketcall,
-            SYS_CONNECT,
-            slice_addr(&[borrwed_fd(fd), by_ref(addr), socklen_t(addrlen), 0, 0, 0]),
-        ))
-    }
-}
-
-#[inline]
-pub(crate) fn shutdown(fd: BorrowedFd<'_>, how: c_int) -> io::Result<()> {
-    #[cfg(not(target_arch = "x86"))]
-    unsafe {
-        ret(syscall2(__NR_shutdown, borrowed_fd(fd), c_int(how)))
+        ret(syscall2(__NR_shutdown, borrowed_fd(fd), c_uint(how)))
     }
     #[cfg(target_arch = "x86")]
     unsafe {
         ret(syscall2_readonly(
             __NR_socketcall,
             SYS_SHUTDOWN,
-            &[borrwed_fd(fd), c_int(how), 0, 0, 0, 0],
+            &[borrwed_fd(fd), c_uint(how), 0, 0, 0, 0],
         ))
     }
 }
@@ -1366,18 +1481,14 @@ pub(crate) fn getsockname(
 }
 
 #[inline]
-pub(crate) fn bind(
-    fd: BorrowedFd<'_>,
-    addr: &__kernel_sockaddr_storage,
-    addrlen: socklen_t,
-) -> io::Result<()> {
+pub(crate) fn bind_un(fd: BorrowedFd<'_>, addr: &sockaddr_un) -> io::Result<()> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
         ret(syscall3_readonly(
             __NR_bind,
             borrowed_fd(fd),
             by_ref(addr),
-            socklen_t(addrlen),
+            size_of::<sockaddr_un>(),
         ))
     }
     #[cfg(target_arch = "x86")]
@@ -1385,7 +1496,154 @@ pub(crate) fn bind(
         ret(syscall2_readonly(
             __NR_socketcall,
             SYS_BIND,
-            slice_addr(&[borrwed_fd(fd), by_ref(addr), socklen_t(addrlen), 0, 0, 0]),
+            slice_addr(&[
+                borrwed_fd(fd),
+                by_ref(addr),
+                size_of::<sockaddr_un>(),
+                0,
+                0,
+                0,
+            ]),
+        ))
+    }
+}
+
+#[inline]
+pub(crate) fn bind_in(fd: BorrowedFd<'_>, addr: &sockaddr_in) -> io::Result<()> {
+    #[cfg(not(target_arch = "x86"))]
+    unsafe {
+        ret(syscall3_readonly(
+            __NR_bind,
+            borrowed_fd(fd),
+            by_ref(addr),
+            size_of::<sockaddr_in>(),
+        ))
+    }
+    #[cfg(target_arch = "x86")]
+    unsafe {
+        ret(syscall2_readonly(
+            __NR_socketcall,
+            SYS_BIND,
+            slice_addr(&[
+                borrwed_fd(fd),
+                by_ref(addr),
+                size_of::<sockaddr_in>(),
+                0,
+                0,
+                0,
+            ]),
+        ))
+    }
+}
+
+#[inline]
+pub(crate) fn bind_in6(fd: BorrowedFd<'_>, addr: &sockaddr_in6) -> io::Result<()> {
+    #[cfg(not(target_arch = "x86"))]
+    unsafe {
+        ret(syscall3_readonly(
+            __NR_bind,
+            borrowed_fd(fd),
+            by_ref(addr),
+            size_of::<sockaddr_in6>(),
+        ))
+    }
+    #[cfg(target_arch = "x86")]
+    unsafe {
+        ret(syscall2_readonly(
+            __NR_socketcall,
+            SYS_BIND,
+            slice_addr(&[
+                borrwed_fd(fd),
+                by_ref(addr),
+                size_of::<sockaddr_in6>(),
+                0,
+                0,
+                0,
+            ]),
+        ))
+    }
+}
+
+#[inline]
+pub(crate) fn connect_un(fd: BorrowedFd<'_>, addr: &sockaddr_un) -> io::Result<()> {
+    #[cfg(not(target_arch = "x86"))]
+    unsafe {
+        ret(syscall3_readonly(
+            __NR_connect,
+            borrowed_fd(fd),
+            by_ref(addr),
+            size_of::<sockaddr_un>(),
+        ))
+    }
+    #[cfg(target_arch = "x86")]
+    unsafe {
+        ret(syscall2_readonly(
+            __NR_socketcall,
+            SYS_CONNECT,
+            slice_addr(&[
+                borrwed_fd(fd),
+                by_ref(addr),
+                size_of::<sockaddr_un>(),
+                0,
+                0,
+                0,
+            ]),
+        ))
+    }
+}
+
+#[inline]
+pub(crate) fn connect_in(fd: BorrowedFd<'_>, addr: &sockaddr_in) -> io::Result<()> {
+    #[cfg(not(target_arch = "x86"))]
+    unsafe {
+        ret(syscall3_readonly(
+            __NR_connect,
+            borrowed_fd(fd),
+            by_ref(addr),
+            size_of::<sockaddr_in>(),
+        ))
+    }
+    #[cfg(target_arch = "x86")]
+    unsafe {
+        ret(syscall2_readonly(
+            __NR_socketcall,
+            SYS_CONNECT,
+            slice_addr(&[
+                borrwed_fd(fd),
+                by_ref(addr),
+                size_of::<sockaddr_in>(),
+                0,
+                0,
+                0,
+            ]),
+        ))
+    }
+}
+
+#[inline]
+pub(crate) fn connect_in6(fd: BorrowedFd<'_>, addr: &sockaddr_in6) -> io::Result<()> {
+    #[cfg(not(target_arch = "x86"))]
+    unsafe {
+        ret(syscall3_readonly(
+            __NR_connect,
+            borrowed_fd(fd),
+            by_ref(addr),
+            size_of::<sockaddr_in6>(),
+        ))
+    }
+    #[cfg(target_arch = "x86")]
+    unsafe {
+        ret(syscall2_readonly(
+            __NR_socketcall,
+            SYS_CONNECT,
+            slice_addr(&[
+                borrwed_fd(fd),
+                by_ref(addr),
+                size_of::<sockaddr_in6>(),
+                0,
+                0,
+                0,
+            ]),
         ))
     }
 }
@@ -1413,7 +1671,7 @@ pub(crate) fn listen(fd: BorrowedFd<'_>, backlog: c_int) -> io::Result<()> {
 #[inline]
 pub(crate) fn sched_yield() {
     unsafe {
-        syscall0_readonly(__NR_sched_yield);
+        let _ = syscall0_readonly(__NR_sched_yield);
     }
 }
 
@@ -1666,6 +1924,11 @@ pub(crate) fn memfd_create(name: &CStr, flags: c_uint) -> io::Result<OwnedFd> {
     unsafe { ret_owned_fd(syscall2(__NR_memfd_create, c_str(name), c_uint(flags))) }
 }
 
+#[inline]
+pub(crate) unsafe fn userfaultfd(flags: c_uint) -> io::Result<OwnedFd> {
+    ret_owned_fd(syscall1(__NR_userfaultfd, c_uint(flags)))
+}
+
 #[inline]
 pub(crate) fn getpid() -> __kernel_pid_t {
     unsafe { syscall0_readonly(__NR_getpid) as __kernel_pid_t }
