@@ -1,16 +1,15 @@
-use crate::path::DecInt;
+use crate::{io, path::DecInt};
 #[cfg(target_os = "hermit")]
-use std::os::hermit::ext::ffi::OsStrExt;
+use std::os::hermit::ext::ffi::{OsStrExt, OsStringExt};
 #[cfg(unix)]
-use std::os::unix::ffi::OsStrExt;
+use std::os::unix::ffi::{OsStrExt, OsStringExt};
 #[cfg(target_os = "vxworks")]
-use std::os::vxworks::ext::ffi::OsStrExt;
+use std::os::vxworks::ext::ffi::{OsStrExt, OsStringExt};
 #[cfg(target_os = "wasi")]
-use std::os::wasi::ffi::OsStrExt;
+use std::os::wasi::ffi::{OsStrExt, OsStringExt};
 use std::{
     borrow::Cow,
     ffi::{CStr, CString, OsStr, OsString},
-    io,
     path::{Component, Components, Iter, Path, PathBuf},
     str,
 };
@@ -34,6 +33,15 @@ pub trait Arg {
     #[cfg(not(windows))]
     fn as_c_str(&self) -> io::Result<Cow<CStr>>;
 
+    /// Consume `self` and teturn a view of this string as a maybe-owned
+    /// [`CStr`].
+    ///
+    /// [`CStr`]: https://doc.rust-lang.org/std/ffi/struct.CStr.html
+    #[cfg(not(windows))]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b;
+
     /// Return a view of this string as a byte slice.
     #[cfg(not(windows))]
     fn as_maybe_utf8_bytes(&self) -> &[u8];
@@ -59,7 +67,20 @@ impl Arg for &str {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -88,7 +109,20 @@ impl Arg for String {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -107,7 +141,7 @@ impl Arg for String {
 impl Arg for &OsStr {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.to_str().ok_or_else(utf8_error)
+        self.to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -118,7 +152,20 @@ impl Arg for &OsStr {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_maybe_utf8_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self.as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -137,7 +184,7 @@ impl Arg for &OsStr {
 impl Arg for OsString {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.as_os_str().to_str().ok_or_else(utf8_error)
+        self.as_os_str().to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -148,7 +195,20 @@ impl Arg for OsString {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_maybe_utf8_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self.into_vec()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -167,7 +227,7 @@ impl Arg for OsString {
 impl Arg for &Path {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.as_os_str().to_str().ok_or_else(utf8_error)
+        self.as_os_str().to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -178,7 +238,20 @@ impl Arg for &Path {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_os_str().as_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_os_str().as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self.as_os_str().as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -197,7 +270,7 @@ impl Arg for &Path {
 impl Arg for PathBuf {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.as_os_str().to_str().ok_or_else(utf8_error)
+        self.as_os_str().to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -208,7 +281,20 @@ impl Arg for PathBuf {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_os_str().as_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_os_str().as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self.into_os_string().into_vec()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -227,7 +313,7 @@ impl Arg for PathBuf {
 impl Arg for &CStr {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.to_str().map_err(|_utf8_err| utf8_error())
+        self.to_str().map_err(|_utf8_err| io::Error::ILSEQ)
     }
 
     #[inline]
@@ -241,6 +327,15 @@ impl Arg for &CStr {
         Ok(Cow::Borrowed(self))
     }
 
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Borrowed(self))
+    }
+
     #[cfg(not(windows))]
     #[inline]
     fn as_maybe_utf8_bytes(&self) -> &[u8] {
@@ -257,7 +352,7 @@ impl Arg for &CStr {
 impl Arg for CString {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.as_c_str().to_str().map_err(|_utf8_err| utf8_error())
+        self.to_str().map_err(|_utf8_err| io::Error::ILSEQ)
     }
 
     #[inline]
@@ -268,7 +363,63 @@ impl Arg for CString {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Borrowed(self.as_c_str()))
+        Ok(Cow::Borrowed(self))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(self))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn as_maybe_utf8_bytes(&self) -> &[u8] {
+        self.as_bytes()
+    }
+
+    #[cfg(windows)]
+    #[inline]
+    fn as_os_str(&self) -> io::Result<Cow<OsStr>> {
+        self.as_ref()
+    }
+}
+
+impl<'a> Arg for Cow<'a, str> {
+    #[inline]
+    fn as_str(&self) -> io::Result<&str> {
+        Ok(self)
+    }
+
+    #[inline]
+    fn to_string_lossy(&self) -> Cow<str> {
+        Cow::Borrowed(self)
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn as_c_str(&self) -> io::Result<Cow<CStr>> {
+        Ok(Cow::Owned(
+            CString::new(self.as_ref()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            match self {
+                Cow::Owned(s) => CString::new(s),
+                Cow::Borrowed(s) => CString::new(s),
+            }
+            .map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -287,7 +438,7 @@ impl Arg for CString {
 impl<'a> Arg for Cow<'a, OsStr> {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        (**self).to_str().ok_or_else(utf8_error)
+        (**self).to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -298,7 +449,24 @@ impl<'a> Arg for Cow<'a, OsStr> {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            match self {
+                Cow::Owned(os) => CString::new(os.into_vec()),
+                Cow::Borrowed(os) => CString::new(os.as_bytes()),
+            }
+            .map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -317,7 +485,7 @@ impl<'a> Arg for Cow<'a, OsStr> {
 impl<'a> Arg for Component<'a> {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.as_os_str().to_str().ok_or_else(utf8_error)
+        self.as_os_str().to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -328,7 +496,20 @@ impl<'a> Arg for Component<'a> {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_os_str().as_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_os_str().as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self.as_os_str().as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -347,7 +528,7 @@ impl<'a> Arg for Component<'a> {
 impl<'a> Arg for Components<'a> {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.as_path().to_str().ok_or_else(utf8_error)
+        self.as_path().to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -358,9 +539,22 @@ impl<'a> Arg for Components<'a> {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(
-            self.as_path().as_os_str().as_bytes(),
-        )?))
+        Ok(Cow::Owned(
+            CString::new(self.as_path().as_os_str().as_bytes())
+                .map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self.as_path().as_os_str().as_bytes())
+                .map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -379,7 +573,7 @@ impl<'a> Arg for Components<'a> {
 impl<'a> Arg for Iter<'a> {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.as_path().to_str().ok_or_else(utf8_error)
+        self.as_path().to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -390,9 +584,22 @@ impl<'a> Arg for Iter<'a> {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(
-            self.as_path().as_os_str().as_bytes(),
-        )?))
+        Ok(Cow::Owned(
+            CString::new(self.as_path().as_os_str().as_bytes())
+                .map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self.as_path().as_os_str().as_bytes())
+                .map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -411,7 +618,7 @@ impl<'a> Arg for Iter<'a> {
 impl Arg for &[u8] {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        str::from_utf8(self).map_err(|_utf8_err| utf8_error())
+        str::from_utf8(self).map_err(|_utf8_err| io::Error::ILSEQ)
     }
 
     #[inline]
@@ -422,7 +629,20 @@ impl Arg for &[u8] {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(*self)?))
+        Ok(Cow::Owned(
+            CString::new(*self).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -441,7 +661,7 @@ impl Arg for &[u8] {
 impl Arg for Vec<u8> {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        str::from_utf8(self).map_err(|_utf8_err| utf8_error())
+        str::from_utf8(self).map_err(|_utf8_err| io::Error::ILSEQ)
     }
 
     #[inline]
@@ -452,7 +672,20 @@ impl Arg for Vec<u8> {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.clone())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_slice()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -471,7 +704,7 @@ impl Arg for Vec<u8> {
 impl Arg for DecInt {
     #[inline]
     fn as_str(&self) -> io::Result<&str> {
-        self.as_os_str().to_str().ok_or_else(utf8_error)
+        self.as_os_str().to_str().ok_or(io::Error::ILSEQ)
     }
 
     #[inline]
@@ -482,7 +715,20 @@ impl Arg for DecInt {
     #[cfg(not(windows))]
     #[inline]
     fn as_c_str(&self) -> io::Result<Cow<CStr>> {
-        Ok(Cow::Owned(CString::new(self.as_os_str().as_bytes())?))
+        Ok(Cow::Owned(
+            CString::new(self.as_os_str().as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
+    }
+
+    #[cfg(not(windows))]
+    #[inline]
+    fn into_c_str<'b>(self) -> io::Result<Cow<'b, CStr>>
+    where
+        Self: 'b,
+    {
+        Ok(Cow::Owned(
+            CString::new(self.as_os_str().as_bytes()).map_err(|_cstr_err| io::Error::ILSEQ)?,
+        ))
     }
 
     #[cfg(not(windows))]
@@ -497,13 +743,3 @@ impl Arg for DecInt {
         self.as_ref()
     }
 }
-
-#[cfg(libc)]
-fn utf8_error() -> io::Error {
-    io::Error::from_raw_os_error(libc::EILSEQ)
-}
-
-#[cfg(linux_raw)]
-fn utf8_error() -> io::Error {
-    io::Error::from_raw_os_error(linux_raw_sys::errno::EILSEQ as i32)
-}
